#ifndef __KYEE_SECURITY_H__
#define __KYEE_SECURITY_H__

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

struct TrustedRootData {
	char runTime[64];           // report time
	int startTimes;
	char kyeeVersion[64];       // 
	int cpuNumber;
	char totalMemory[64];       // 总内存(MB)
	char availableMemory[64];   // 可用内存(M)
	char totalSpace[64];        // flash存储空间(M)
	char availableSpace[64];    // flash可用存储空间(M)

	int measureMode;            // 0:No 1:Init 2:Measure
	int measureNums;            // 度量基准库条数
	int measureTimes;           // 启动度量次数
	int bootTrustedState;       // 启动可信状态（0-可信；1-不可信；2-未度量；）
	int bootMode;               // 启动模式（0-同步模式；0x13-混合模式；0x23-异步模式；）
};

struct BootMeasureData{
	char time[64];    //度量时间
	char pname[256]; //度量进程名
	bool measure_result;
	int start_measure_error_number;    
};

// 与tee_api_types.h中保持一致
typedef struct {
	uint32_t seconds;
	uint32_t millis;
} TEE_Time;

struct BootMeasureLog {
	uint32_t numbers;
	struct BootMeasureData report[32];
} ;

struct KYEE_TrustChainData {
	int tpcm;
	int uefi;
	int grub;
	char time[64];
	int  totalnum;
	int  failnum;
};

typedef enum 
{
	kyee_tpcm = 0,
	kyee_uefi = 1,
	kyee_grub = 2,
	kyee_bios = 3
}KyeeReportOpt;


#ifdef __cplusplus
extern "C" {
#endif

	// 返回值：0成功，其他值失败
	int KYEE_GetTrustedRootData(struct TrustedRootData *data);

	// 获取当前系统启动度量策略状态
	//0 关闭； 1 初始化；2 开启； 3 警告； 其他 失败
	int KYEE_GetBootMeasureStatus();

	// 获取系统配置启动度量策略状态
	//0 关闭； 1 初始化；2 开启； 3 警告； 其他 失败
	int KYEE_GetBootMeasureCfgStatus();

	// 设置启动度量策略状态
	//0 关闭； 2 开启； 3 警告    //初始化移除
	int KYEE_SetBootMeasureStatus(int status);

	// 设置启动度量策略状态
	//0 关闭； 2 开启； 3 警告    //初始化移除
	int KYEE_SetBootMeasureCfgStatus(int status);

	// 设置重新采集度量值
	int KYEE_SetReCollectMeasureStandarValue(int status);

	// 获取重新采集度量值
	int KYEE_GetReCollectMeasureStandarValue();

	int KYEE_GetBootMeasureData(struct BootMeasureData **list, int *count);

	/**
	 * @brief 获取当前启动报告
	 * 
	 * @param reportopt 需要获取的报告类型（0.tpcm, 1.uefi 2.grub）
	 * @param list 当前启动报告数据 
	 * @param count 数据数量
	 * @return 0， 成功；其他失败
	 */
	int KYEE_GetBootMeasureData_v2(int reportopt, struct BootMeasureData **list, int *count);

	// 获取启动模式
	//0 同步； 1 混合；2 异步； 其他 失败
	int KYEE_GetBootMode();

	// 设置启动模式
	// status： 设置状态， 0 同步 1 混合 2 异步
	int KYEE_SetBootMode(int status);

	//flash启动日志存储
	int KYEE_SaveFlashBootMeasureLog();

	//获取历史启动报告
	int KYEE_GetHistoryBootMeasureLog(struct BootMeasureLog **list, int *count);

	/**
	 * @brief获取可信链的度量结果
	 * 
	 * @param data 可信链数据 (out)   
	 *
	 * @return 0， success, 
	 *         other, failed
	 */
	int KYEE_GetTrustChainData(struct KYEE_TrustChainData* data);

	int KYEE_AddFile();
	int KYEE_DealFile();


#ifdef __cplusplus
}
#endif

#endif

